/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

import jess.*;
import java.awt.Point;
  
public class HandlerRobot implements JessListener
{
   Robot robot;

   public HandlerRobot(Robot robot)
   {
      super();
      this.robot = robot;
   }

   public void eventHappened(JessEvent je)
   {
      int defaultMask = JessEvent.DEFRULE_FIRED | JessEvent.FACT |
                        JessEvent.RESET | JessEvent.CLEAR;
      int type = je.getType();

      Rete engine = (Rete) je.getSource();
      int mask = engine.getEventMask();
      mask |= defaultMask;
      engine.setEventMask(mask);

      switch (type)
      {
         case JessEvent.CLEAR:               
         engine = (Rete) je.getSource();
         mask = engine.getEventMask();
         mask |= defaultMask;
         engine.setEventMask(mask);
         break;

         case JessEvent.RESET:
            break;
          
         case JessEvent.DEFRULE_FIRED:
            break;

         case JessEvent.HALT:
            // Colocar aviso de robot parado
            robot.desenharTermino();
            break;

         case JessEvent.RUN:
            break;
          
         case JessEvent.FACT | JessEvent.REMOVED:
            break;

         case JessEvent.FACT:

            int x, // posio x na tela
                y; // posicao y na tela, casados formam a posicao (x,y)
              
            Point posicao; // ponto que representa a posicao atual
            Point obstaculo;
            Point visitado;

            String face; // direo apontada pela frente do robot [<,>,V,^]
            int obstaculo_x, obstaculo_y;
            int visitado_x, visitado_y;
            int chave;

            try
            {
               Value comando = engine.fetch("verificar-possibilidades");
               chave = comando.intValue(engine.getGlobalContext());
               switch (chave)
               {
                  case 1:
                     // Leitura dos dados da base de conhecimento
                     Value px = engine.fetch("posicao-x");
                     Value py = engine.fetch("posicao-y");
                     Value fce = engine.fetch("face");
                     Value obs_x = engine.fetch("obs-x");
                     Value obs_y = engine.fetch("obs-y");
                     Value vis_x = engine.fetch("vis-x");
                     Value vis_y = engine.fetch("vis-y");
                     
                     // Converso dos dados para Strings vlidas
                     x = px.intValue(engine.getGlobalContext());
                     y = py.intValue(engine.getGlobalContext());
                     face = fce.stringValue(engine.getGlobalContext());
                     obstaculo_x = obs_x.intValue(engine.getGlobalContext());
                     obstaculo_y = obs_y.intValue(engine.getGlobalContext());
                     visitado_x = vis_x.intValue(engine.getGlobalContext());
                     visitado_y = vis_y.intValue(engine.getGlobalContext());

                     posicao = new Point(x,y);
                     obstaculo = new Point(obstaculo_x, obstaculo_y);
                     visitado = new Point(visitado_x, visitado_y);

                     // Ordens de ao ao robot

                     robot.virar(face);               
                     robot.andar(posicao);
                     robot.marcarVisitado(visitado);
                     robot.limpar(obstaculo);
                     break;
                  case 2:
                     // O robot pede ao agente centralizador prioridade para esta
                     // tarefa dizendo que ela no pode ser interrompida...

                     Value tmp_x = engine.fetch("posicao-x");
                     Value tmp_y = engine.fetch("posicao-y");
   
                     x = tmp_x.intValue(engine.getGlobalContext());
                     y = tmp_y.intValue(engine.getGlobalContext());
                     posicao = new Point(x ,y);
                     // Faz a verificao das possibilidades de movimento dado uma
                     // posio atual
                     robot.verificacao(posicao);
                     break;
                  case 3:
                     // No  necessrio mais alta prioridade
                     robot.standByOff();
                     break;
                  default:
                     break;
               }
            }
            catch (JessException jex)
            {
              // Impresso dos erros encontrados
              System.err.println("ERRO! HANDLER ROBOT");
            }
      }
   }
}