/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

import jess.*;
import java.awt.Point;
  
public class HandlerRobot implements JessListener
{
   Robot robot;

   public HandlerRobot(Robot robot)
   {
      super();
      this.robot = robot;
   }

   public void eventHappened(JessEvent je)
   {
      int defaultMask = JessEvent.DEFRULE_FIRED | JessEvent.FACT |
                        JessEvent.RESET | JessEvent.CLEAR;
      int type = je.getType();

      Rete engine = (Rete) je.getSource();
      int mask = engine.getEventMask();
      mask |= defaultMask;
      engine.setEventMask(mask);

      switch (type)
      {
         case JessEvent.CLEAR:               
         engine = (Rete) je.getSource();
         mask = engine.getEventMask();
         mask |= defaultMask;
         engine.setEventMask(mask);
         break;

         case JessEvent.RESET:
            break;
          
         case JessEvent.DEFRULE_FIRED:
            break;

         case JessEvent.HALT:
            robot.ativado = false;
            break;

         case JessEvent.RUN:
            robot.ativado = true;
            break;
          
         case JessEvent.FACT | JessEvent.REMOVED:
            break;

         case JessEvent.FACT:

            int x, // posio x na tela
                y; // posicao y na tela, casados formam a posicao (x,y)
              
            Point posicao, // ponto que representa a posicao atual
                  obstaculo; // refere-se ao obstaculo vencido

            String face; // direo apontada pela frente do robot [<,>,V,^] 
            int vx, vy;

            try
            {
               // Leitura dos dados da base de conhecimento
               Value px = engine.fetch("posicao-x");
               Value py = engine.fetch("posicao-y");
               Value fce = engine.fetch("face");
               Value vx1 = engine.fetch("obs-x");
               Value vy1 = engine.fetch("obs-y");
               
               // Converso dos dados para Strings vlidas
               x = px.intValue(engine.getGlobalContext());
               y = py.intValue(engine.getGlobalContext());
               face = fce.stringValue(engine.getGlobalContext());
               vx = vx1.intValue(engine.getGlobalContext());
               vy = vy1.intValue(engine.getGlobalContext());

               posicao = new Point(x,y);
               obstaculo = new Point(vx, vy);

               // Ordens de ao ao robot

               robot.virar(face);
               robot.andar(posicao);
               robot.limpar(obstaculo);
            }
            catch (JessException jex)
            {
              // Impresso dos erros encontrados
              System.err.println(jex);
            }
            break;
      }
   }
}