/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

// Esta classe se comportar como se fosse o agente centralizador 
// dos robots. Toda a sua comunicao grfica ser ouvida pela classe 
// HandlerAgente.java

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import jess.*;

public class AgenteCentralizador extends JFrame
{
   Graphics g;
   private Container c;
   private JMenuBar bar;
   // Area de desenho do ambiente
   private AreaGrafica areaG;
   // Configura o ambiente
   private ConfigAgente configAg;
   // Numero de robots no ambiente
   private int numeroRobots;
   // Guarda os estados iniciais dos robots
   private Vector posicoesIniciais;
   private Vector obstaculosVencidos;
   private Vector jaVisitados;
   // Tempo pelo qual o processador deve ser atrasado para que 
   // um usurio possa ver as modificaes nos estados com mais
   // tranqilidade
   private long tempoEspera;
   // Os dados de um robot devem ser guardados dentro
   // de um container para que possam ser armazenados
   // consistentemente
   private ContainerEstados estadosRobots[];
   // Vetor de robots
   private Robot bots[];
   // Limite da gui o que corresponde a uma matriz 7x7 obviamente 
   // com 49 posies de movimentaes diferentes possveis
   private int limiteGUI = 7;
   // Chave para verificar se os robots j esto em estado 
   // consistente para uma inicializao e operao corretas
   public boolean robotsConfigurados;
   public boolean acaoCompleta;
   public Vector obstaculos;
   
   // Construtor padro para o AgenteCentralizador
   public AgenteCentralizador()
   {
      super("Robots Semi Conscientes [Conhecem apenas a si prprios]");
      posicoesIniciais = new Vector();
      obstaculosVencidos = new Vector();
      obstaculos = new Vector();

      c = getContentPane();
      // Criando a barra de menus
      bar = new JMenuBar();
      // Configura a barra de menus para o JFrame
      setJMenuBar(bar);

      // Nenhum robot configurado
      robotsConfigurados = false;

      criarMenus();
      anexarAreaGrafica();

      //setSize(306, 356);
      setSize(426, 476);
      // Este frame no pode ser redimensionada
      setResizable(false);
      // Cor de fundo  branco
      setBackground(Color.white);
      // Exibindo o frame na tela      
      show();
   }

   // Anexa um painel grfico onde sero exibidas as deliberaes
   // sobre a base de conhecimento bem como 
   // as modificaes no ambiente de trabalho
   public void anexarAreaGrafica()
   {
      areaG =  new AreaGrafica();
      Color corFundo = new Color(79, 175, 209);
      areaG.setBackground(corFundo);
      c.add(areaG, BorderLayout.CENTER);
   }

   // Cria o menu superior do programa
   private void criarMenus()
   {
      // --------------------- M e n u  A r q u i v o ----------------------------//
      // Cria a barra de menus
      JMenu menuArquivo = new JMenu("Arquivo");
      // Escolhendo o mnemonico para a opo
      menuArquivo.setMnemonic('A');

      JMenuItem itemSobre = new JMenuItem("About...");
      itemSobre.setMnemonic('A');
      itemSobre.addActionListener(new HandlerAgente(AgenteCentralizador.this, this));
      menuArquivo.add(itemSobre);
      menuArquivo.addSeparator();

      JMenuItem itemSair = new JMenuItem("Sair");
      itemSair.setMnemonic('S');
      itemSair.addActionListener(new HandlerAgente(AgenteCentralizador.this, this));
      menuArquivo.add(itemSair);
      bar.add(menuArquivo);

      // ---------------------- M e n u  C o n f i g -----------------------------//
      JMenu menuCaracteristicas = new JMenu("Caracteristicas");
      menuCaracteristicas.setMnemonic('C');

      JMenuItem itemConfig = new JMenuItem("Config");
      itemConfig.setMnemonic('C');
      itemConfig.addActionListener(new HandlerAgente(AgenteCentralizador.this, this));
      menuCaracteristicas.add(itemConfig);

      JMenuItem itemCriarRobot = new JMenuItem("Criar e executar Robot(s)");
      itemCriarRobot.setMnemonic('R');
      itemCriarRobot.addActionListener(new HandlerAgente(AgenteCentralizador.this, this));
      menuCaracteristicas.add(itemCriarRobot);
      bar.add(menuCaracteristicas);
   }

   // Cria o painel de configuaes onde ser lida do usurio
   // caractersticas do ambiente e dos robots
   public void setCaracteristicas()
   {
      posicoesIniciais = new Vector();
      obstaculosVencidos = new Vector();
      obstaculos = new Vector();
      
      configAg = new ConfigAgente(this, limiteGUI);
   }

   // Seleciona o tempo de movimentao e deliberao
   // dos robots
   public void setTempoEspera(long tempo)
   {
      tempoEspera = tempo;
      areaG.setTempoEspera(tempo);       
   }
   
   // Seta o numero de robots que faro as deliberaes no ambiente
   // Quanto mais robots mais rpido a tarefa pode ser realizada
   public void setNumeroRobots(int nroRobots)
   {
      numeroRobots = nroRobots;
      criarContainerEstados();
      jaVisitados = new Vector(numeroRobots);
   }

   public void setPosicaoInicial(Point pto)
   {
      posicoesIniciais.addElement(pto);
   }

   public void criarRobot()
   {
      // Limpando para execuo
      areaG.limparTudo();

      int sorteio;
      Point pto;
      bots = new Robot[numeroRobots];
      // Criando os robots
      for (int i = 0; i < numeroRobots; i++)
      {
         sorteio = (int)(1 + Math.random() * 4);
         bots[i] = new Robot(this, ("robot" + i), i, (long)(1 + Math.random() * 10));
         if (posicoesIniciais.elementAt(i) != null)
         {
            pto = (Point)(posicoesIniciais.elementAt(i));
            switch (sorteio)
            {
               case 1:
                  estadosRobots[i].setPosicao(pto);
                  estadosRobots[i].setFace('n');
                  bots[i].setEstadoInicial(pto, "norte");
                  break;
               case 2:
                  estadosRobots[i].setPosicao(pto);
                  estadosRobots[i].setFace('s');
                  bots[i].setEstadoInicial(pto, "sul");
                  break;
               case 3:
                  estadosRobots[i].setPosicao(pto);
                  estadosRobots[i].setFace('o');
                  bots[i].setEstadoInicial(pto, "oeste");                  
                  break;
               default:
                  estadosRobots[i].setPosicao(pto);
                  estadosRobots[i].setFace('l');
                  bots[i].setEstadoInicial(pto, "leste");                  
                  break;
            }               
         }
         else
         {
            pto = new Point(i, i);
           switch (sorteio)
            {
               case 1:
                  bots[i].setEstadoInicial(pto, "norte");                  
                  break;
               case 2:
                  bots[i].setEstadoInicial(pto, "sul");                  
                  break;
               case 3:
                  bots[i].setEstadoInicial(pto, "oeste");                  
                  break;
               default:
                  bots[i].setEstadoInicial(pto, "leste");                  
                  break;
            }               
         }
      }

      // Desenhando os obstaculos
      setObstaculos();

      // Iniciando os robots
      for (int i = 0; i < bots.length; i++)
      {
         bots[i].start();
      }
   }

   // Criado para armazenar os estados de cada robot
   // modus que o AgentEcentralizador saiba a o estado atual de
   // cada um destes
   private void criarContainerEstados()
   {
      estadosRobots = new ContainerEstados[numeroRobots];

      for (int i = 0; i < estadosRobots.length; i++)
         estadosRobots[i] = new ContainerEstados();
   }

   
   public void limpar(Point pos, int id)
   {
      if (obstaculosVencidos.indexOf(pos) == -1)
      {
         obstaculosVencidos.addElement(pos);
         areaG.pintarLimpeza(pos);
      }
   }

   public void setVisitado(Point pos, int id)
   {
      if (jaVisitados.indexOf(pos) == -1)
      {
         jaVisitados.addElement(pos);
         jaVisitados.removeElement(estadosRobots[id].getVisitado());
         estadosRobots[id].adicionarVisitado(pos);
         // Pode vir a ser utilizado futuramente
         areaG.pintarVisitado(estadosRobots[id].getVisitado());
      }
   }

   public void setObstaculos()
   {
      for (int i = 0; i < obstaculos.size(); i++)
      {
         areaG.pintarObstaculo((Point)(obstaculos.elementAt(i)));
      }
   }

   public void adicionarObstaculo(Point pto)
   {
      if (obstaculos.indexOf(pto) == -1)
      {
         obstaculos.addElement(pto);
      }
   }
   
   public void setFace(char ch, int id)
   {
      if (estadosRobots[id].getPosicao() != null)
      {
         estadosRobots[id].setFace(ch);
         areaG.pintar(estadosRobots[id].getPosicao(), estadosRobots[id].getFace());
      }
   }

   public void setFinal(Point pos, int id)
   {
      areaG.pintar(pos, estadosRobots[id].getFace());
   }  
 
   public void setPosicao(Point pos, int id)
   {
      Point pto = new Point();
      pto.setLocation(estadosRobots[id].getPosicao());
      estadosRobots[id].setPosicao(pos);
      areaG.pintar(estadosRobots[id].getPosicao(), estadosRobots[id].getFace());
      areaG.limpar(pto);
   }

   public synchronized boolean existeVisitado(Point pto, int id)
   {
      for (int i = 0; i < jaVisitados.size(); i++)
      {
         if (jaVisitados.elementAt(i).equals(pto))
            return true;
      }

      // Sem esta instruo o robot anda eternamente pelo ambiente
      // procurando soluo para o problema.
      if (estadosRobots[id].temVisitado(pto))
         return true;
      
      return false;
   }

   public synchronized void verificarPossibilidades(Point pto, int id)
   {
      int x = (int)(pto.getX());
      int y = (int)(pto.getY());
      int possibilidades[] = new int[4];
      Point aux = new Point();
      String comando = "", comando1 = "";

      for (int i = 0; i < 4; i++)
      {
         switch (i)
         {
            case 0:
               if (y+1 < 7)
               {
                  aux.setLocation(x, y+1);                  
                  if (existeVisitado(aux, id))
                     possibilidades[i] = 0;
                  else
                     possibilidades[i] = 1;
               }
               else
                  possibilidades[i] = 0;
               break;
            case 1:
               if (y-1 >= 0)
               {
                  aux.setLocation(x, y-1);                  
                  if (existeVisitado(aux, id))
                     possibilidades[i] = 0;   
                  else
                     possibilidades[i] = 1;   
               }
               else
                  possibilidades[i] = 0;
               break;
            case 2:
               if (x+1 < 7)
               {
                  aux.setLocation(x+1, y);                  
                  if (existeVisitado(aux, id))
                     possibilidades[i] = 0;   
                  else
                     possibilidades[i] = 1;   
               }
               else
                  possibilidades[i] = 0;
               break;
            default:
               if (x-1 >= 0)
               {
                  aux.setLocation(x-1, y);                  
                  if (existeVisitado(aux, id))
                     possibilidades[i] = 0;   
                  else
                     possibilidades[i] = 1;   
               }
               else
                  possibilidades[i] = 0;
               break;
         }
      }

      try
      {
         bots[id].robot.executeCommand("(sorteiar " + 
              possibilidades[0] + " " + possibilidades[1] + " " +
              possibilidades[2] + " " + possibilidades[3] + ")");           

         Value tmp = bots[id].robot.fetch("retorno");
         int valor = tmp.intValue(bots[id].robot.getGlobalContext());
         int px, py;
         Point novaPosicao;
         switch (valor)
         {
            case 1:
               py = y + 1;
               novaPosicao = new Point(x, py);
               setVisitado(novaPosicao, id);
               break;
            case 2:
               py = y - 1;
               novaPosicao = new Point(x, py);
               setVisitado(novaPosicao, id);
               break;
            case 3:
               px = x + 1;
               novaPosicao = new Point(px, y);
               setVisitado(novaPosicao, id);
               break;
            case 4:
               px = x - 1;
               novaPosicao = new Point(px, y);
               setVisitado(novaPosicao, id);
               break;
         }   
      }
      catch (JessException jex)
      {
         System.err.println(jex);
      }
   }
   
   // Responsvel por chamar o programa
   public static void main(String args[])
   {
      AgenteCentralizador agente = new AgenteCentralizador();

      agente.addWindowListener( new WindowAdapter()
      {
         public void windowClosing(WindowEvent e)
         {
            System.exit(0);
         }
      });
   }
}