/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

// Esta classe  responsvel pela leitura das caractersticas 
// adicionais do ambiente

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class ConfigAgente extends JFrame
{
   private Container c;
   private JLabel labelAvisos;
   private JLabel labelRobots;
   private JLabel labelEspera;
   private JLabel labelNumeroObstaculos;
   public JTextField numeroRobots;
   public JTextField tempoEspera;
   public JTextField status;
   public JButton setupCompleto;
   public JTextField numeroObstaculos;
   // Ponteiro para saber que est sendo configurado
   private AgenteCentralizador agente;
   private int limiteGUI;

   // Construtor default quando chamado cria um painel 
   // onde sero lidos as caractersticas do ambiente
   public ConfigAgente(AgenteCentralizador ag, int limiteGUI)
   {
      super("Configurando o ambiente e os ROBOTs");
      
      agente = ag;
      this.limiteGUI = limiteGUI;
      c = getContentPane();
      c.setLayout(new FlowLayout());

      labelAvisos = new JLabel("Comeando as configuraes...");
      labelRobots = new JLabel("Nmero de Robots no ambiente ");
      labelEspera = new JLabel("Tempo de espera (mili-seg)   ");
      labelNumeroObstaculos = new JLabel("Numero de obstaculos");
      numeroRobots = new JTextField(10);
      tempoEspera = new JTextField(10);
      numeroObstaculos = new JTextField(10);
      status = new JTextField(45);
      setupCompleto = new JButton("<< Retornar <<");
      
      status.setText("Ambiente ainda sem ROBOTs");
      status.setEditable(false);
      status.setFont(new Font("Courier", Font.ITALIC + Font.BOLD, 11));
      status.setBackground(Color.green);

      tempoEspera.setEnabled(false);
      setupCompleto.setEnabled(false);
      numeroObstaculos.setEnabled(false);
      
      labelAvisos.setText("Ao digitar uma opo pressione ENTER");
      
      // Anexando os ouvintes da GUI que servem para ouvir qualquer modificao 
      // no estado de um desses componentes
      numeroRobots.addActionListener(new HandlerConfigAgente(ConfigAgente.this, this));
      tempoEspera.addActionListener(new HandlerConfigAgente(ConfigAgente.this, this));
      setupCompleto.addActionListener(new HandlerConfigAgente(ConfigAgente.this, this));
      numeroObstaculos.addActionListener(new HandlerConfigAgente(ConfigAgente.this, this));
      
      // Anexando componentes ao container grfico
      c.add(labelAvisos); 
      c.add(labelRobots);
      c.add(numeroRobots);
      c.add(labelEspera);
      c.add(tempoEspera);
      c.add(labelNumeroObstaculos);
      c.add(numeroObstaculos);
      c.add(status);
      c.add(setupCompleto);
      
      // Janela de configurao tem tamanho fixo
      setResizable(false);
      setSize(400, 250);
      show();
   }

   // Seta o tempo de espera. Este tempo  o tempo entre uma deliberao 
   // e outra pelos robots. Quanto maior o tempo mais lenta a execuo das
   // tarefas passadas
   public void setTempoEspera(long tempo)
   {
      agente.setTempoEspera(tempo);
   }

   // Setando o numero de robots que trabalharo no ambiente
   public void setNumeroRobots(int nroRobots)
   {
      agente.setNumeroRobots(nroRobots);
   }

   public void setPosicaoInicial(Point pto)
   {
      agente.setPosicaoInicial(pto);
   }
   
   public void setObstaculo(Point pto)
   {
      agente.adicionarObstaculo(pto);
   }
   
   public int getLimite()
   {
      return limiteGUI;
   }
}