/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

// Esta classe  responsvel pela leitura das caractersticas 
// adicionais do ambiente

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class ConfigRobot extends JFrame
{
   private JLabel labelSujeira;
   private int numeroSujeiras;
   private Container c;
   private JTextField textNumeroSujeiras;
   private JTextField textTempoEspera;
   private JTextField textAviso;
   private JTextField status;
   private JButton setupCompleto;
   private JLabel labelTempoEspera;
   private GUIBot gui;
   
   public ConfigRobot(GUIBot gui)
   {
      super("Configurando ROBOTs");
      
      this.gui = gui;
      c = getContentPane();
      c.setLayout(new FlowLayout());
      labelSujeira = new JLabel("Nmero de obstculos para o Robot");
      status = new JTextField(45);
      labelTempoEspera = new JLabel("Tempo (mili-segundos)");
      status.setText("Obstculos ainda no definidos");
      status.setEditable(false);

      status.setFont(new Font("Courier", Font.ITALIC + Font.BOLD, 11));
      textNumeroSujeiras = new JTextField(10);
      textAviso = new JTextField(25);
      setupCompleto = new JButton("<< Retornar <<"); 
      textAviso.setEditable(false);
      textAviso.setBackground(Color.green);
      textAviso.setText("Ao digitar uma opo pressione ENTER");
      textTempoEspera = new JTextField(10);
      textTempoEspera.setEnabled(false);      

      textNumeroSujeiras.addActionListener(new ConfigRobotHandler(ConfigRobot.this));
      setupCompleto.addActionListener(new ConfigRobotHandler(ConfigRobot.this));
      textTempoEspera.addActionListener(new ConfigRobotHandler(ConfigRobot.this));
      setupCompleto.setEnabled(false);

      c.add(textAviso);
      c.add(labelSujeira);
      c.add(textNumeroSujeiras);
      c.add(labelTempoEspera);
      c.add(textTempoEspera);
      c.add(status);
      c.add(setupCompleto);
      
      setResizable(false);
      setSize(400, 250);
      show();
   }

   private void setSujeira(Point pto)
   {
      if (gui.sujeiras.indexOf(pto) == -1)
      {
         gui.sujeiras.addElement(pto);
      }
   }

   public int getLimite()
   {
      return gui.getLimitesGUI();
   }

   private void setNumeroSujeiras(int ns)
   {
      if (ns > 0)
      {
         numeroSujeiras = ns;
      }
   }

   public int getNumeroSujeiras()
   {
      return numeroSujeiras;
   }

   // ******************** Classe Tratadora de eventos ******************** //

   private class ConfigRobotHandler implements ActionListener
   {
      Component componentePai;
      String entrada = "";

      ConfigRobotHandler(Component cpt)
      {
         componentePai = cpt;
      }

      public void actionPerformed(ActionEvent e)
      {
         if (e.getSource() == setupCompleto)
         {
            dispose();
         }
         else if (e.getSource() == textTempoEspera)
         {
            long tempo;
            try
            {
               tempo = (Long.parseLong(e.getActionCommand()));
               gui.setTempoEspera(tempo);
               status.setText("Para concluir clique em RETORNAR");
               setupCompleto.setEnabled(true);
               textTempoEspera.setEnabled(false);
            }
            catch (NumberFormatException nfe)
            {
               JOptionPane.showMessageDialog(componentePai, "Tempo invlido. Tempo padro escolhido", 
                                            "Aviso", JOptionPane.WARNING_MESSAGE);
               gui.setTempoEspera(250);
               setupCompleto.setEnabled(true);
            }
         }
         else if (e.getSource() == textNumeroSujeiras)
         {
            try
            {
               setNumeroSujeiras(Integer.parseInt(e.getActionCommand()));
               lerDados(getNumeroSujeiras());
               textNumeroSujeiras.setEnabled(false);
               status.setText("Digite o tempo de movimentao do ROBOT");
               textTempoEspera.setEnabled(true);
            }
            catch (NumberFormatException nfe)
            {
               JOptionPane.showMessageDialog(componentePai, "Digite posies vlidas!\n" + 
                                            "Ex.: 3,2 [Posio x=3, y=2]\n" + 
                                            "Repita a operao!",
                                            "Aviso", JOptionPane.WARNING_MESSAGE);
               textNumeroSujeiras.setText("");
            }
         }
      }

      // Nova funo de leitura de dados
      private void lerDados(int limite)
      {
         String pNumero = "", sNumero = "";
         char tmp;
         boolean primeiro = true;
         int pontoX, pontoY;

         for (int i = 1; i <= limite; i++)
         {
            try
            {
               entrada = JOptionPane.showInputDialog(componentePai, "Entre com a posio do Obstculo e clique OK\n" +
                                                    "Ex.: 2,3 quer dizer [x=2, y=3]\n" +
                                                    "Este  o obstculo " + i + " de " + limite + 
                                                    "\nNO pressione ENTER");

               for (int j = 0; j < entrada.length(); j++)
               {
                  tmp = entrada.charAt(j);
                  if (tmp != ',')
                  {
                     if (primeiro)
                        pNumero += tmp;
                     else
                        sNumero += tmp;
                  }
                  else
                     primeiro = false;
               }
               primeiro = true;

               pontoX = Integer.parseInt(pNumero);
               pontoY = Integer.parseInt(sNumero);
               
               // Verificando se o ponto passado  valido
               if (pontoX >= 0 && pontoX < getLimite() && pontoY >= 0 && pontoY < getLimite())
               {
                  // Adicionando ponto vlido  lista de obstculos
                  Point ptoSujeira = new Point(pontoX, pontoY);
                  setSujeira(ptoSujeira);
               }
               pNumero = "";
               sNumero = "";
            }
            catch(NullPointerException npe)
            {
               JOptionPane.showMessageDialog(componentePai, "Passo " + i + " cancelado." + 
                                             "\nIndo para o prximo passo", "Passo cancelado", 
                                             JOptionPane.WARNING_MESSAGE);
            }
         }
      }
   }
}