/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import jess.*;
import java.util.*;

public class GUIBot extends JFrame
{
   Graphics g;
   // Este  o robot propriamente dito
   public Robot robot;  
   // rea de contedo na tela
   private Container c;
   // Tipo de leiaute do programa
   // Recebe a limitao da GUI a ser exibida
   private int limitesGUI = 7;
   private JMenuBar bar;
   public int pontoX, pontoY;
   public Painel painelDesenho;
   public ConfigRobot config;
   public Vector sujeiras;
   public boolean robotConfigurado;
   private KqmlServidor servidor = new KqmlServidor(this);

   public GUIBot()
   {
      super("Robot com comunicao KQML");
      
      servidor.start();

      robotConfigurado = false;
      c = getContentPane();
      // Criando a barra de menus
      bar = new JMenuBar();
      // Configura a barra de menus para o JFrame
      setJMenuBar(bar);

      criarMenus();
      anexarPainelDesenho();

      setSize(426, 476);
      // Este frame no pode ser redimensionada
      setResizable(false);
      // Cor de fundo  branco
      setBackground(Color.white);
      // Exibindo o frame na tela      
      show();
   }

   public void anexarPainelDesenho()
   {
      painelDesenho =  new Painel();
      Color corFundo = new Color(79, 175, 209);
      painelDesenho.setBackground(corFundo);
      c.add(painelDesenho, BorderLayout.CENTER);
   }
   
   public void setVisitado(Point pto)
   {
      painelDesenho.exibirVisitado(pto);
   }

   // Trocando a visualizao
   public void setFace(char fc)
   {
      switch (fc)
      {
         case 'n':
            painelDesenho.setFace('s');   
            break;
         case 's':
            painelDesenho.setFace('n');   
            break;
         default :
            painelDesenho.setFace(fc);   
            break;
      }
   }
   
   public void setEstado(Point pto)
   {
      pontoX = (int)pto.getX();
      pontoY = (int)pto.getY();
      painelDesenho.draw(pontoX, pontoY);
   }

   public void exibirLimpeza(Point pto)
   {
      painelDesenho.limparPto(pto);
   }
   
   public void exibirSujeiras()
   {
      Point ptoSujeira;

      for (int i = 0; i < sujeiras.size(); i++)
      {
         ptoSujeira = (Point)sujeiras.elementAt(i);
         painelDesenho.exibirSujeira(ptoSujeira);
      }
   }

   private void criarMenus()
   {
      // Cria a barra de menus
      JMenu menuArquivo = new JMenu("Arquivo");
      // Escolhendo o mnemonico para a opo
      menuArquivo.setMnemonic('A');

      JMenuItem itemSobre = new JMenuItem("About...");
      itemSobre.setMnemonic('A');
      itemSobre.addActionListener(new ItemHandler(GUIBot.this, this));
      menuArquivo.add(itemSobre);
      menuArquivo.addSeparator();

      JMenuItem itemSair = new JMenuItem("Sair");
      itemSair.setMnemonic('S');
      itemSair.addActionListener(new ItemHandler(GUIBot.this, this));
      menuArquivo.add(itemSair);
      bar.add(menuArquivo);

      // -------------- Start --------------- //
      ImageIcon iconStart = new ImageIcon("menuRobot.GIF");
      JMenuItem itemStart = new JMenuItem("Start");
      itemStart.setIcon(iconStart);
      itemStart.setMnemonic('S');
      itemStart.addActionListener(new ItemHandler(GUIBot.this, this));
      bar.add(itemStart);

      // -------------- Menu Robot --------------- //
      ImageIcon iconGraficos = new ImageIcon("menuGraficos.GIF");
      // Cria a barra de menus
      JMenu menuGraficos = new JMenu("Graficos");
      menuGraficos.setIcon(iconGraficos);
      menuGraficos.setMnemonic('G');
      // Um item de menu
      
      JMenuItem itemLimparGUI = new JMenuItem("Limpar GUI");
      itemLimparGUI.setMnemonic('L');
      itemLimparGUI.addActionListener(new ItemHandler(GUIBot.this, this));
      menuGraficos.add(itemLimparGUI);

      bar.add(menuGraficos);

      // -------------- Menu dados Iniciais --------------- //
      JMenuItem itemDadosIniciais = new JMenuItem("Setup");
      itemDadosIniciais.setMnemonic('t');
      itemDadosIniciais.addActionListener(new ItemHandler(GUIBot.this, this));
      bar.add(itemDadosIniciais);
   }

   // Retorna o tamanho da GUI que deve ser multiplicado por ele prprio
   // para obter o nmero de posies corretas
   public int getLimitesGUI()
   {
      return limitesGUI;
   }

   public void setTempoEspera(long tempo)
   {
      painelDesenho.setTempoEspera(tempo);
   }
   
   public void inicializarRobot()
   {
      try
      {
         robot = new Robot(this);
         robot.iniciarRobot();
      }
      catch(JessException jex)
      {
         System.err.println(jex);
      }
   }

   public void executarComando(String comando, Point pto, char face)
   {
       setFace(face);
       if (comando.equals("setEstado"))
       {
           setEstado(pto);
       }
       else if (comando.equals("setVisitado"))
           setVisitado(pto);
   }

   // Mtodo principal que chama o programa  execuo
   public static void main(String args[])
   {
      GUIBot bot = new GUIBot();

      bot.addWindowListener( new WindowAdapter()
      {
         public void windowClosing(WindowEvent e)
         {
            System.exit(0);
         }
      });
   }
}

class ItemHandler implements ActionListener
{
   Component componentePai;
   private GUIBot bot;

   ItemHandler(Component cpt, GUIBot bt)
   {
      componentePai = cpt;
      bot = bt;
   }

   public void actionPerformed(ActionEvent e)
   {
      String comando = e.getActionCommand();
      if (comando.equals("Start"))
      {
         if (bot.robotConfigurado)
         {
            bot.painelDesenho.limpar();
            bot.inicializarRobot();
            bot.robotConfigurado = false;
         }
         else
         {
            JOptionPane.showMessageDialog(componentePai, "ROBOT ainda no configurado" + 
                                          "\nV em \"Setup\" para faz-lo", "Aviso",
                                          JOptionPane.WARNING_MESSAGE);      
         }
      }
      else if (comando.equals("About..."))
      {
         ImageIcon iconeAbout = new ImageIcon("itemAbout.GIF");
         JOptionPane.showMessageDialog(componentePai, "Projeto de Iniciao Cientfica" +
                           " - PIBIC/CNPq\n\n" + "Bolsista: Anderson de Rezende Rocha\n" +
                           "Orientador: Antnio Maria Pereira de Resende\n" + 
                           "\nDPTO . DE CINCIA DA COMPUTAO - UFLA, 2002", 
                           "Sobre o projeto", 1, iconeAbout);
      }
      else if (comando.equals("Sair"))
      {
         System.exit(0);
      }
      else if (comando.equals("Limpar GUI"))
      {
         bot.painelDesenho.limpar();
         bot.robotConfigurado = false;
      }
      else if (comando.equals("Setup"))
      {
         // Criao do Frame de setup
         bot.config = new ConfigRobot(bot);
         bot.sujeiras = new Vector();
         bot.robotConfigurado = true;
      }
   }
}