/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

import jess.*;
import java.awt.Point;
  
public class HandlerRobot implements JessListener
{
   Robot robot;

   public HandlerRobot(Robot robot)
   {
      super();
      this.robot = robot;
   }

   public void eventHappened(JessEvent je)
   {
      int defaultMask = JessEvent.DEFRULE_FIRED | JessEvent.FACT |
                        JessEvent.RESET | JessEvent.CLEAR;
      int type = je.getType();

      Rete engine = (Rete) je.getSource();
      int mask = engine.getEventMask();
      mask |= defaultMask;
      engine.setEventMask(mask);

      switch (type)
      {
         case JessEvent.CLEAR:               
         engine = (Rete) je.getSource();
         mask = engine.getEventMask();
         mask |= defaultMask;
         engine.setEventMask(mask);
         break;

         case JessEvent.RESET:
            break;
          
         case JessEvent.DEFRULE_FIRED:
            break;

         case JessEvent.HALT:
            robot.ativado = false;
            break;

         case JessEvent.RUN:
            robot.ativado = true;
            break;
          
         case JessEvent.FACT | JessEvent.REMOVED:
            break;

         case JessEvent.FACT:

            int x, // posio x na tela
                y, // posicao y na tela, casados formam a posicao (x,y)
                visitadoX, // posicao vitada X
                visitadoY, // posicao visitado Y, modus que [visX,visY] 
                           // demonstra uma posio cartesiana visitada
                limpoX,
                limpoY;
               
            Point posicao, // ponto que representa a posicao atual
                  ptoLimpo, // ponto representando sujeira retirada
                  visitado; // ponto representado uma regiao j visitada

            String face; // direo apontada pela frente do robot [<,>,V,^]

            try
            {
               // Leitura dos dados da base de conhecimento
               Value px = engine.fetch("posicao-x");
               Value py = engine.fetch("posicao-y");
               Value fce = engine.fetch("face");
               Value vx = engine.fetch("vis-x");
               Value vy = engine.fetch("vis-y");
               Value limpoSx = engine.fetch("limpo-x");
               Value limpoSy = engine.fetch("limpo-y");
               
               // Converso dos dados para Strings vlidas
               x = px.intValue(engine.getGlobalContext());
               y = py.intValue(engine.getGlobalContext());
               limpoX = limpoSx.intValue(engine.getGlobalContext());
               limpoY = limpoSy.intValue(engine.getGlobalContext());
               visitadoX = vx.intValue(engine.getGlobalContext());
               visitadoY = vy.intValue(engine.getGlobalContext());
               face = fce.stringValue(engine.getGlobalContext());

               posicao = new Point(x,y);
               visitado = new Point(visitadoX,visitadoY);
               ptoLimpo = new Point(limpoX, limpoY);

               // Ordens de ao ao robot

               robot.setLimpeza(ptoLimpo);
               robot.virar(face);               
               robot.andar(posicao);
               robot.setVisitado(visitado);
            }
            catch (JessException jex)
            {
              // Impresso dos erros encontrados
              System.err.println(jex);
            }
            break;
      }
   }
}