/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

// Esta classe  o Gerenciador Grafico do ambiente. Apenas uma  instanciada

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import jess.*;
import java.rmi.*;
import java.rmi.server.*;

public class GGrafRobotImpl extends UnicastRemoteObject implements GGrafRobot
{
   private Vector botsIPs = new Vector();
   private Vector botsIDs = new Vector();
   public Vector obstaculos = new Vector();
   
   public String hostName = "127.0.0.1";
   public boolean execucaoLocal;
   public boolean jaRegistrado;
   public boolean setupCompleto = false;
   static private GGrafRobotImpl gGraf;

   // Servidor Grafico executando Local ou Remotamente
   private MyJFrame frame;
   Graphics g;
   private Container c;
   private JMenuBar bar;
   private AreaGrafica areaG;
   private ConfigGraf config;

   // Escolhendo o tamanho da matriz de exibicao no ambiente
   private final int LIMITESGUI = 10;
   
   // Cada robot instanciado registra-se aqui de modo que passe seu IP 
   // e receba o IP de possveis companheiros
   public Vector apresentacao(String ip, String id) throws RemoteException
   {
      botsIDs.addElement(id);

      avisarOutrosRobots(ip, id);
      botsIPs.addElement(ip);
      return botsIPs;
   }

   public Vector getIDs()
   {
      return botsIDs;
   }
   
   public GGrafRobotImpl(String ip, String status) throws RemoteException
   {
      // Verifica o tipo de execuao;
      if (status.equals("local"))
         execucaoLocal = true;
      else if (status.equals("remoto"))
         execucaoLocal = false;

      jaRegistrado = false;

      hostName = ip;
      frame = new MyJFrame("Projeto de Robots Distribuidos");

      c = frame.getContentPane();
      // Criando a barra de menus
      bar = new JMenuBar();
      // Configura a barra de menus para o JFrame
      frame.setJMenuBar(bar);

      criarMenus();
      anexarAreaGrafica();

      frame.setSize(426, 476);
      // Este frame no pode ser redimensionada
      frame.setResizable(false);
      // Exibindo o frame na tela      
      frame.show();
   }

   public void criarMenus()
   {
      // --------------------- M e n u  A r q u i v o ----------------------------//
      // Cria a barra de menus
      JMenu menuArquivo = new JMenu("Arquivo");
      // Escolhendo o mnemonico para a opo
      menuArquivo.setMnemonic('A');

      JMenuItem itemSobre = new JMenuItem("About...");
      itemSobre.setMnemonic('A');
      itemSobre.addActionListener(new HandlerGraf(this));
      menuArquivo.add(itemSobre);
      menuArquivo.addSeparator();

      JMenuItem itemSair = new JMenuItem("Sair");
      itemSair.setMnemonic('S');
      itemSair.addActionListener(new HandlerGraf(this));
      menuArquivo.add(itemSair);
      bar.add(menuArquivo);

      // ---------------------- M e n u  C o n f i g -----------------------------//
      JMenu menuCaracteristicas = new JMenu("Caracteristicas");
      menuCaracteristicas.setMnemonic('C');

      JMenuItem itemConfig = new JMenuItem("Config...");
      itemConfig.setMnemonic('C');
      itemConfig.addActionListener(new HandlerGraf(this));
      menuCaracteristicas.add(itemConfig);

      JMenuItem itemCriarRobot = new JMenuItem("Criar e executar Robot(s)");
      itemCriarRobot.setMnemonic('R');
      itemCriarRobot.addActionListener(new HandlerGraf(this));
      menuCaracteristicas.add(itemCriarRobot);
      bar.add(menuCaracteristicas);
   }

   public void anexarAreaGrafica()
   {
      areaG =  new AreaGrafica();
      Color corFundo = new Color(79, 175, 209);
      areaG.setBackground(corFundo);
      c.add(areaG, BorderLayout.CENTER);
   }

   public void setCaracteristicas()
   {
      config = new ConfigGraf(this, LIMITESGUI);
      // Estou dizendo que a partir desse momento ja posso receber
      // robots em meu ambiente
   }
   
   public void registrarIP()
   {
      try
      {
         // Estou me registrando
         Naming.rebind("//"+hostName+"/GGrafRobot", gGraf);
         System.out.println("Estou registrado! (GGrafRobot)");
         jaRegistrado = true;
      }
      catch(Exception e)
      {
         System.err.println("Sistema remoto nao encontrado. Tentando conectar novamente");
      }
   }

   public Vector getObstaculos() throws RemoteException
   {
      if (setupCompleto)
      {
         setupCompleto = false;
         return obstaculos;         
      }
      else
      {
         areaG.desenharObstaculos(obstaculos);
         return obstaculos;         
      }
   }
   
   public void adicionarObstaculo(Point pto)
   {
      obstaculos.addElement(pto);
   }

   public void setTempoEspera(long tempo)
   {
      areaG.setTempoEspera(tempo);
   }

   public void desenharPosicao(Point pto, char fc)
   {
      areaG.pintarMovimento(pto, fc);
   }

   public void desenharVisitado(Point pto) throws RemoteException
   {
      areaG.pintarVisitado(pto);
   }
   
   public void desenharLimpeza(Point pto) throws RemoteException
   {
      areaG.pintarLimpeza(pto);
   }

   public void terminarEsteBot(Point pto, char fc, String id, String ip) throws RemoteException
   {
      areaG.pintarMovimento(pto, fc);
   }

   // Avisa aos outros robots que outro companheiro chegou no ambiente
   private void avisarOutrosRobots(String ip, String id)
   {
      boolean contato = false;
      String host = "";
      String nomeBot;


      for (int i = 0; i < botsIPs.size(); i++)
      {
         contato = false;
         while (!contato)
         {
            try 
            {
               host = (String)(botsIPs.elementAt(i));
               nomeBot = (String)(botsIDs.elementAt(i));
               Robot bot = (Robot)(Naming.lookup("//"+host+"/"+nomeBot));
               contato = true;
               bot.inserirCompanheiro(ip, id);
            } 
            catch (Exception ex) 
            {
               contato = false;
               System.err.println("Contato nao estabelecido");
            }
         }
      }
   }
   
   // Responsvel por chamar o programa
   public static void main(String args[])
   {
      try
      {
         gGraf = new GGrafRobotImpl(args[0], args[1]);
         gGraf.registrarIP();

         gGraf.frame.addWindowListener( new WindowAdapter()
         {
            public void windowClosing(WindowEvent e)
            {
               System.exit(0);
            }
         });
      }
      catch(Exception e)
      {
         System.err.println(e);
      }
   }
}